/// <reference types="node" />
import { PackageJson, TsConfigJson } from "type-fest";
interface Options {
    /** @default utf-8 */
    encoding: BufferEncoding;
    /**
     * Will throw in case of FS error or invalid JSON
     * @default true
     *  */
    throws: boolean;
    /** @default "throw" (even if throws is false) */
    ifFieldIsMissing: "throw" | "skip" | "add";
    /**
     * - throw - throws (even if throws param is false)
     *-  skip - won't call the function
     * - pass - pass the `undefined` value
     * @default "throw"
     * */
    ifFieldIsMissingForSetter: "throw" | "skip" | "pass";
    /**
     * - null - disable formatting
     * - hard - one hard tab \t
     * - number - number of spaces
     * @default "preserve"
     *  */
    tabSize: null | number | "preserve" | "hard";
}
declare type GettersDeep<T extends object> = {
    [K in keyof T]: (oldValue: T[K], json: T) => unknown;
};
export declare type ModifyJsonFileFunction<T extends object> = (path: string, fields: Partial<T | GettersDeep<T>>, options?: Options) => Promise<void>;
declare type ModifyJsonFileGenericFunction = <T extends object>(path: string, fields: Partial<T | GettersDeep<T>>, options?: Partial<Options>) => Promise<void>;
/**
 * modifies **original** JSON file
 * You can pass generic, that reflects the structure of original JSON file
 *
 * Fields, that are functions will be skipped if they're not preset in original JSON file
 */
export declare const modifyJsonFile: ModifyJsonFileGenericFunction;
/**
 * Almost the same is sindresorhus/write-pkg, but with proper typing support and setters for fields
 */
export declare const modifyPackageJson: ModifyJsonFileFunction<PackageJson>;
export declare const modifyTsConfigJson: ModifyJsonFileFunction<TsConfigJson>;
export {};
