"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.modifyTsConfigJson = exports.modifyPackageJson = exports.modifyJsonFile = void 0;
const fs_1 = __importDefault(require("fs"));
const detect_indent_1 = __importDefault(require("detect-indent"));
const load_json_file_1 = __importDefault(require("load-json-file"));
/** It's just Error */
class InnerError extends Error {
    constructor() {
        super(...arguments);
        this.innerError = true;
    }
}
/**
 * modifies **original** JSON file
 * You can pass generic, that reflects the structure of original JSON file
 *
 * Fields, that are functions will be skipped if they're not preset in original JSON file
 */
const modifyJsonFile = async (path, fields, options) => {
    const { encoding = "utf-8", throws = true, ifFieldIsMissing = "throw", ifFieldIsMissingForSetter = "throw", tabSize: tabSizeOption = "preserve" } = options || {};
    try {
        const json = await load_json_file_1.default(path);
        // todo remove restriction or not?
        if (!json || typeof json !== "object" || Array.isArray(json))
            throw new TypeError(`${path}: JSON root type must be object`);
        // todo we don't need to read the file twice
        const rawText = await fs_1.default.promises.readFile(path, encoding);
        const indent = tabSizeOption === "preserve" ? detect_indent_1.default(rawText).indent : tabSizeOption === "hard" ? "\t" : tabSizeOption === null ? undefined : " ".repeat(tabSizeOption);
        for (const [name, value] of Object.entries(fields)) {
            if (!(name in json)) {
                const isSetter = typeof value === "function";
                const generalAction = isSetter ? ifFieldIsMissingForSetter : ifFieldIsMissing;
                if (generalAction === "throw")
                    throw new InnerError(`Property to modify "${name}" is missing in ${path}`);
                if (generalAction === "skip")
                    continue;
            }
            json[name] = typeof value === "function" ? value(json[name], json) : value;
        }
        await fs_1.default.promises.writeFile(path, JSON.stringify(json, undefined, indent));
    }
    catch (err) {
        if (err.innerError)
            throw new Error(err.message);
        if (throws)
            throw err;
    }
};
exports.modifyJsonFile = modifyJsonFile;
// todo: use read-pkg / write-pkg for normalization
/**
 * Almost the same is sindresorhus/write-pkg, but with proper typing support and setters for fields
 */
exports.modifyPackageJson = exports.modifyJsonFile;
exports.modifyTsConfigJson = exports.modifyJsonFile;
